/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/



#include <QString>
#include <QSettings>

#include "bsdportglobal.h"
#include "runcmd.h"
#include "ftpindexhandler.h"
#include "vulnadbhandler.h"
#include "portsdirhandler.h"

//#include <sys/param.h>
#include <sys/utsname.h>

#include <math.h>

/******** QBsdPortGlobal ****************/

/** private **/
QString QBsdPortGlobal::randomString(int length)
{
/* TODO fix this
    QString ret("");
    qsrand(QTime::currentTime().second());

    for (int i = 0; i < length; ++i)
        ret.append(QChar(qrand()%26)+65);

    return ret;*/
    return "random";
}

QString QBsdPortGlobal::getMakeEnv( const QString& path,
                                    const QString& var,
                                    const QStringList& env )
{
    QStringList list;
    return getMakeEnv(path, list+=var, env);
}

QString QBsdPortGlobal::getMakeEnv( const QString& path,
                                    const QStringList& vars,
                                    const QStringList& env )
{
    if (!QFile::exists(path+"/Makefile"))
    {
        emit warning("getMakeEnv() : Directory \"" + path +
                     "\" contains no Makefile, cant get Var." );
        return "";
    }

//    QString cmd = "cd \"" + path + "\" && " + env.join(" ") + " make -V " + var;
//    return QBsdPort::global->runCmd(cmd,1).at(0);
    QString cmd = "/usr/bin/make -C " + path + " ";
    foreach (QString var, vars)
        cmd += "-V " + var + " ";
    cmd += env.join(" ");

    return QBsdPort::global->runCmd(cmd).join(" ");
}

QStringList QBsdPortGlobal::runCmd(const QString& cmd, int linesOfOutput)
{
    /** this call runs a command in foreground and reads #linesOufOutput,
        where 0 means as much output as there is **/

//    qDebug(cmd.toLatin1());
    FILE*   pipe;
    char    buffer[10001] = "";
    QString qbuffer("");
    QStringList lbuffer;
    //TODO remove frame, not neccessary
//    QString realcmd = "/bin/sh -c \'" + cmd + " 2>&1\'";
    QString realcmd = cmd + " 2>&1";

    if( (pipe = popen( realcmd.toLatin1(), "r" )) == NULL )
    {
           emit error("runCmd() : could not open pipe for command:\n" + cmd);
           return QStringList();
    }

    while (!feof(pipe) &&
            ((lbuffer.count() < linesOfOutput) || (!linesOfOutput)) )
    {
        fgets( buffer, 10000, pipe );
        qbuffer = buffer;
        qbuffer.remove(qbuffer.length()-1,1);
        qbuffer.remove('\n');
        lbuffer.append(qbuffer);
    }


    /// remove trailing empty lines
    while ( (!lbuffer.isEmpty()) &&
            ( (lbuffer.last() == "") || (lbuffer.last() == " ") ) )
        lbuffer.removeLast();

    /// work around the double last line
    if (lbuffer.count() >1 )
        if (lbuffer[lbuffer.count()-1] == lbuffer[lbuffer.count()-2])
            lbuffer.removeLast();

    if (lbuffer.isEmpty())
        lbuffer.append("");

    pclose(pipe);

    return lbuffer;
}

void QBsdPortGlobal::readSettings()
{
    QSettings settings(PREFIX "/etc/bpk.conf", QSettings::NativeFormat);

    utsname* un = new utsname();
    if (uname(un) != 0) 
        emit critical("Could not get uname-Information from sys/utsname.h");

    QString freeBSDVersion = un->release;
    freeBSDVersion = freeBSDVersion.toLower();

    //BUG: detecting release version doesnt seem to work
    qDebug("UNAME: " + freeBSDVersion.toLatin1());
    int majorVersion = freeBSDVersion.contains('.') ?
                        freeBSDVersion.section('.', 0, 0).toInt() :
                        freeBSDVersion.section('-', 0, 0).toInt();
    int minorVersion = freeBSDVersion.contains('.') ?
                freeBSDVersion.section('-', 0, 0).section('.', 1, 1).toInt() :
                        -1;
    QString release = freeBSDVersion.section('-', 1, 1);
    if ((release == "stable") || (release == "current") )
        minorVersion = -1;
    else if (release != "release") {}
//        emit warning("You seem to be running neither -RELEASE nor -STABLE nor"
//                     " -CURRENT...");
     
    
    QString defVal;

    if (!m_portsDir.cd(settings.value("portsDir", "/usr/ports/").toString()))
        emit error("PortsDir does not exist! Change the Path in the Settings"
                   " or refetch Ports");


    if (!m_pkgDbDir.cd(settings.value("pkgDbDir", "/var/db/pkg/").toString()))
        emit error("PkgDb-Directory does not exist! "

              "Change the Path in the Settings.");


    if (!QFile::exists(portsDir().path() +
                      "/INDEX-" + QString::number(majorVersion)))
        emit error("INDEX-File not found. Change the Path in the Settings"
                   " or refetch Portstree");
    else
        defVal = portsDir().path()+"/INDEX-" + QString::number(majorVersion);

    indexFile().setFileName(settings.value("indexFile", defVal).toString());


    QString defaultPkgMirror = "ftp://ftp.freebsd.org/pub/FreeBSD/ports/";

    QString arch = un->machine;

    defVal = getenv("PACKAGESITE") ? getenv("PACKAGESITE")
                                   : defaultPkgMirror + arch + "/packages-" +
                                     QString::number(majorVersion) +
                                     (minorVersion == -1 ? "" :
                                       "." + QString::number(minorVersion)) +
                                     "-" + release + "/Latest/";
    m_pkgSite = settings.value("pkgSite", defVal).toString();

    //TODO connect this to settings as well
    m_pkgDirs.clear();
    m_pkgDirs.append("/usr/ports/Packages/All");


    m_bigPkgSize = settings.value("bigPkgSize", 1024*1024*50 /*50MiB*/).toInt();

    m_updDate = QDate::fromString( settings.value("updDate",
                                                  "2008-01-01").toString(),
                                  Qt::ISODate);

    //TODO connect this to settings as well
    m_defaultBuildEnv.clear();
    m_defaultBuildEnv.append("BATCH=1");
    m_defaultBuildEnv.append("DISABLE_VULNERABILITIES=1");

    #ifdef PORTAUDIT
    //TODO
    setPortaudit(1);
    #endif


}

void QBsdPortGlobal::writeSettings()
{
//     QMutexLocker locker(&settingsLock);
    QSettings settings("/usr/local/etc/bpk.conf", QSettings::NativeFormat);

    settings.setValue("portsDir", portsDir().absolutePath());
    settings.setValue("pkgDbDir", pkgDbDir().absolutePath());
    settings.setValue("pkgSite", pkgSite());
    settings.setValue("updDate", updDate().toString("yyyy-MM-dd"));
    settings.setValue("indexFile", indexFile().fileName());
    settings.setValue("bigPkgSize", bigPkgSize());


    settings.sync();
    //TODO connect defaultbuildenv,pkgDirs 

}

/*
void QBsdPortGlobal::defaultValues()
{
    //TODO: check environment vars and os-specific consts for these vars

    if (!m_portsDir.cd("/usr/ports/"))
        m_portsDir.cd("illegal");

//    if (!m_portsDbDir.cd("/var/db/ports/"))
//        m_portsDbDir.cd("illegal");

    if (!m_pkgDbDir.cd("/var/db/pkg/"))
        m_pkgDbDir.cd("illegal");


    if (QFile::exists("/usr/ports/INDEX-7"))
        m_indexFile.setFileName("/usr/ports/INDEX-7");
    else
        m_indexFile.setFileName("illegal");

//    if (QFile::exists("/var/db/bpk/FTPINDEX"))

//    else
//        m_ftpIndexFile.setFileName("illegal");

    m_pkgSite = "ftp://ftp.freebsd.org/pub/FreeBSD/ports/i386/"
                "packages-7-stable/Latest/";
    m_pkgDirs.clear();
    m_pkgDirs.append("/usr/ports/Packages/All");
//    m_updDate = 0;

    m_defaultBuildEnv.clear();
    m_defaultBuildEnv.append("BATCH=1");
    m_defaultBuildEnv.append("DISABLE_VULNERABILITIES=1");



    m_bigPkgSize = 1024*1024*50; // 50MB

    m_portaudit = 0; // no portaudit support, not activated

}*/
#ifndef SPONTANEOUS
void QBsdPortGlobal::createErrPkgs()
{
    /// initialize err pkgs here, THESE ARE NEVER CHANGED
    m_errorPort = new QBsdPort("illegal/illegal");
    QBsdPortAvSrcPkg *avsrc = new QBsdPortAvSrcPkg();
    QBsdPortAvBinPkg *avbin = new QBsdPortAvBinPkg();
    QBsdPortInstPkg *inst   = new QBsdPortInstPkg();


    avsrc->setVersion("illegal");
    avsrc->setName("illegal");
//    avsrc->setPrefix("illegal");
    avsrc->setComment("illegal");
    avsrc->setMaint("illegal");
    avsrc->setCategories("illegal");
    avsrc->setWWW("illegal");
//    avsrc->setReqs(QStringList());
    avsrc->setDescr("illegal");
//    avsrc->setFiles(QStringList());
    avsrc->setBroken("illegal");
    avsrc->setIgnore("illegal");
    avsrc->setRestricted("illegal");
    avsrc->setDeprecated("illegal");
    avsrc->setForbidden("illegal");

    avbin->setVersion("illegal");
    avbin->setName("illegal");
//    avbin->setPrefix("illegal");
    avbin->setComment("illegal");
    avbin->setMaint("illegal");
    avbin->setCategories("illegal");
    avbin->setWWW("illegal");
//    avbin->setReqs(QStringList());
    avbin->setDescr("illegal");
//    avbin->setFiles("illegal");

    inst->setVersion("illegal");
    inst->setName("illegal");
//    inst->setPrefix("illegal");
    inst->setComment("illegal");
    inst->setMaint("illegal");
    inst->setCategories("illegal");
    inst->setWWW("illegal");
//    inst->setReqs(QStringList());
    inst->setDescr("illegal");
//    inst->setFiles(QStringList());

    inst->setDeps(QStringList());
//    inst->setMessage("illegal");
//     inst->setUpdWarning("illegal");
    inst->setFalseFiles(QStringList());
#ifdef PORTAUDIT
/*    inst->setVulna("illegal");
    avbin->setVulna("illegal");
    avsrc->setVulna("illegal");*/
#endif
    errorPort()->setAvSrc(avsrc);
    errorPort()->setAvBin(avbin);
    errorPort()->setInst(inst);
}
#endif //NSPONTANEOUS

QBsdPortGlobal::QBsdPortGlobal()
{
    m_collection = new QPortsCollection();

    m_instPkgDbReader = new QInstPkgDbReader();
    m_ftpIndex = new QFtpIndexHandler();
    m_vulnaDbHandler = new QVulnaDbHandler();
    m_portsDirHandler = new QPortsDirHandler();
#ifdef SPONTANEOUS
    init();
#endif
}

QBsdPortGlobal::~QBsdPortGlobal()
{
    writeSettings();

    delete m_errorPort;
    delete m_instPkgDbReader;

}

#ifndef SPONTANEOUS
void    QBsdPortGlobal::getGoing()
{
    start();

    // these objects have to be referencable before to the main thread 
    // forks therfore we push them to the bsdportglobal-thread before it
    // itself is started
    m_collection->moveToThread(this);
    m_instPkgDbReader->moveToThread(this);
    m_ftpIndex->moveToThread(this);
    m_vulnaDbHandler->moveToThread(this);
    m_portsDirHandler->moveToThread(this);
    
    this->moveToThread(this);
//     m_collection.moveToThread(this);
//     QTimer::singleShot(0, this, SLOT(init()));
    QMetaObject::invokeMethod(this, "init", Qt::QueuedConnection);
}
#endif

void QBsdPortGlobal::init()
{
    m_collection->setParent(this);
    m_instPkgDbReader->setParent(this);
    m_ftpIndex->setParent(this);
    m_vulnaDbHandler->setParent(this);
    m_portsDirHandler->setParent(this);

    /// register metadatatypes
    qRegisterMetaType<QBsdPort>();
    qRegisterMetaType<QBsdPort*>("QBsdPort*");
    qRegisterMetaType<QPortList>("QPortList");
    qRegisterMetaType<QBsdPortPkg>();
    qRegisterMetaType<QBsdPortPkg*>("QBsdPortPkg*");

    /// set vars to default
//    defaultValues();
    readSettings();

    /// check for config file
/*    if (QFile::exists("/usr/local/etc/bpk.conf"))
        m_bpkConf.setFileName("/usr/local/etc/bpk.conf");
    else if (QFile::exists("/var/db/bpk/bpk.conf"))
        m_bpkConf.setFileName("/var/db/bpk/bpk.conf");
    else
        m_bpkConf.setFileName("illegal");
*/
    /// if config file exists read it and set vars accordingly
    //TODO: implement

    /// create temporary dirctory
    QString r = randomString(6);
    //TODO check existance of QDir::tempPath+"/bpk-"+randomString(6) and if
    // exists remove dir recursively
    m_sessionDir.mkpath(QDir::tempPath() + "/bpk-" + r);
    m_sessionDir.cd(QDir::tempPath() + "/bpk-" + r);


    createErrPkgs();

    /// create collection
    findAllAvSrc();
  #ifndef SPONTANEOUS

    connect(m_ftpIndex, SIGNAL(updateSuccessful()),
            m_ftpIndex, SLOT(read()));
    m_ftpIndex->update();

    m_instPkgDbReader->start();


  #ifdef PORTAUDIT
    m_vulnaDbHandler->update();
  #endif //portaudit


  #endif //n spontaneous


}

#ifndef SPONTANEOUS
void QBsdPortGlobal::run()
{
    exec();
}
#endif

void QBsdPortGlobal::reset()
{
    collection()->clear();
    findAllAvSrc();
    m_ftpIndex->read();
    m_instPkgDbReader->start();
}



void QBsdPortGlobal::findAllAvSrc()
{
    emit findAllAvSrcRunning();

  #ifdef SPONTANEOUS
    /** in spontaneous mode we iterate through the portsdir and create ports **/
    QStringList cats = getMakeEnv( portsDir().absolutePath(),
                                                          "SUBDIR" ).split(' ');
    QStringList children;
    QString catname(""), childname(""); /// first and second half of origin
    QBsdPort    *p;

    for (int i = 0; i < cats.count(); ++i)
    {
        catname = cats.at(i);
        children = getMakeEnv( portsDir().absolutePath() +"/"+catname,
                                                          "SUBDIR" ).split(' ');

        for (int ii = 0; ii < children.count(); ++ii)
        {
            qDebug("loop:" + (catname+"/"+children.at(ii)).toLatin1());
            p = new QBsdPort(catname+"/"+children.at(ii));
            p->setAvSrc(new QBsdPortAvSrcPkg(p));
            p->avSrc()->setParent(p);

            collection()->append(p);
        }
    }
  #else
    /** in normal mode we read the index file **/
    if (!indexFile().exists())
    {
        emit error("findAllAvailableSrc() : INDEX not found");
        emit findAllAvSrcFinished(0);
        return;
    }

    if (!indexFile().open(QFile::ReadOnly))
    {
        emit error("findAllAvailableSrc() : INDEX found, but not readable");
        emit findAllAvSrcFinished(0);
        return;
    }

//    char                buffer[15000]="";
    QString             qbuffer("");
    qbuffer.reserve(16000);
    QStringList         lbuffer, reqs;
    qint64              lineLength=0;
    QBsdPort            *port;
    QBsdPortAvSrcPkg    *pkg;


    uint counter = 0;
    while (!indexFile().atEnd())
    {
//        lineLength = indexFile().readLine(buffer, sizeof(buffer));

//        if (lineLength == -1) /// error reading from INDEX
//        {
//            emit critical("findAllAvailableSrc() : unknown read error in INDEX");
//            indexFile().close();
//            return;
//        }

        //qbuffer = buffer;
        qbuffer = indexFile().readLine().data();
        if (!qbuffer.endsWith('\n'))
            emit warning("findAllAvailableSrc() : a line not completely read.");
        else
            qbuffer.remove('\n');

        lbuffer = qbuffer.split('|');
        if (lbuffer.size() < 10)
        {
            emit warning(QString("findAllAvailableSrc() : the following INDEX line is not "
                         "in expected format:\n" + qbuffer).toLatin1());
            continue;
        }

        port    = new QBsdPort(lbuffer.at(1).section('/',3,4));
        pkg     = new QBsdPortAvSrcPkg();

        pkg->setVersion(lbuffer.at(0).section('-',-1,-1));
        pkg->setName(lbuffer.at(0).section('-',0,-2));
        pkg->setComment(lbuffer.at(3));
        pkg->setMaint(lbuffer.at(5));
        pkg->setCategories(lbuffer.at(6));
        pkg->setWWW(lbuffer.at(9));


//        reqs.clear();
//        reqs = lbuffer.at(7).split(' ') + lbuffer.at(8).split(' ');
//        pkg->setReqs(removeDuplicates(reqs));

         collection()->append(port);
//          qDebug(qbuffer.toLatin1());
//          qDebug(collection()->at(collection()->size()-1)->origin().toLatin1());

        ++counter;

        port->setAvSrc(pkg);

    }
    indexFile().close();

    collection()->sort();

  #endif //not SPONTANEOUS

    emit findAllAvSrcFinished(counter);

}
