/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QtConcurrentRun>
#include <QFuture>
#include <QApplication>

#include "ftpindexhandler.h"
#include "bsdportglobal.h"
#include "portscollection.h"


void QFtpIndexHandler::update()
{
//     updateProc = new QProcess(this);
//     connect(updateProc, SIGNAL(finished(int, QProcess::ExitStatus)),
//             this,       SLOT(updateFinished(int, QProcess::ExitStatus)));
//     connect(updateProc, SIGNAL(started()),
//             this,       SIGNAL(updateStarted()));
//     QString cmd = "uma -v " + QString::number(QApplication::applicationPid()) +
//                   " fetch ftpindex";
//     updateProc->start(cmd, QIODevice::ReadOnly);

    emit updateStarted();
    QString cmd = "uma -v " + QString::number(QApplication::applicationPid()) +
               " fetch ftpindex";
    updateLog = tr("COMMAND RUN:\n") + cmd + tr("\nRETURN VALUE: %1"
             "\nLOG:\n");
//     updateProc =
    RunCmd::bg(cmd, this, SLOT(processOneOutputLine(const QString&)),
               SLOT(updateFinished(int, QProcess::ExitStatus)));
}

void QFtpIndexHandler::processOneOutputLine(const QString& line)
{
    updateLog += line + "\n";
}

void QFtpIndexHandler::updateFinished(int exitCode,
                                    QProcess::ExitStatus exitStatus)
{
    if (exitStatus)
        emit updateFailed(updateLog);
    else if (exitCode)
        emit updateFailed(updateLog.arg(exitCode));
    else
        emit updateSuccessful();
}

void QFtpIndexHandler::read()
{
    file = new QFile("/var/db/uma/FTPINDEX", this);

    if (!file->exists())
    {
        emit readFailed(tr("FTPINDEX doesn't exist"));
        return;
    }

    if (!file->open(QFile::ReadOnly))
    {
        emit readFailed(tr("FTPINDEX found, but not readable."));
        return;
    }

    counter = 0;

    timer = new QTimer(this);
    connect(timer,  SIGNAL(timeout()),
            this,   SLOT(readOneFileLine()));
    timer->start(0);
    emit readStarted();
}

void QFtpIndexHandler::readOneFileLine()
{
    if (!file->atEnd())
    {
        
//        char    buffer[15000]="";
        QString qbuffer("");
//        qint64  lineLength=0;
//        lineLength = file->readLine(buffer, sizeof(buffer)-1);
//        if (lineLength == -1) /// error reading from INDEX
//        {
//            emit readFailed("unknown error reading from in FTPINDEX");
//            file->close();
//            return;
//        }
        qbuffer = file->readLine().data();
        //TODO: check if the lines read are actually removed from mem again
        QFuture<void> future = QtConcurrent::run(this,
                                           &QFtpIndexHandler::processOneFileLine,
                                           qbuffer);
//         processOneFileLine(qbuffer);
    } else
    {
        timer->stop();
        timer->deleteLater();
        emit readSuccessful(counter);

    }
        

}

void QFtpIndexHandler::processOneFileLine(const QString& line)
{
//     qDebug("foo");
    QStringList         lbuffer,
                        reqs;
    QBsdPortAvBinPkg    *pkg;
    QString             line2 = line;
 
    if (!line2.endsWith('\n'))
    {
        qWarning("QFtpIndexHandler::processOneLine a line not completely read.");

    } else
        line2.remove('\n');

    lbuffer = line2.split('|');

    if (lbuffer.count() < 10)
    {
        qWarning("QFtpIndexHandler::processOneLine a line not completely read. A Pkg skipped.");
        qWarning(line2.toLatin1());
        return;
    }

    QString origin =  lbuffer.at(1).section('/',3,4);
    if (!origin.contains('/'))
        origin = "NO_ORIGIN/" + lbuffer.at(0).section('-',0,-2);

    pkg     = new QBsdPortAvBinPkg();

    pkg->setVersion(lbuffer.at(0).section('-',-1,-1));
    pkg->setName(lbuffer.at(0).section('-',0,-2));
    pkg->setComment(lbuffer.at(3));
    pkg->setMaint(lbuffer.at(5));
    pkg->setCategories(lbuffer.at(6));
    pkg->setWWW(lbuffer.at(9));

    pkg->moveToThread(QBsdPort::global);

    if (! QMetaObject::invokeMethod(
                QBsdPort::global->collection(),
                "addPkgToPort",
    //                         Qt::BlockingQueuedConnection,
                Qt::QueuedConnection,
                Q_ARG(const QString, origin),
                Q_ARG(QBsdPortPkg*, qobject_cast<QBsdPortPkg*>(pkg)) ) )
        qDebug("could not invoke method port");
//     QBsdPort::global->collection()->addPkgToPort(origin, pkg);
    
    counter.ref();
}

void QFtpIndexHandler::clear()
{
    foreach(QBsdPort* p, QBsdPort::global->collection()->collection())
    {
        if (p->hasAvBin())
            p->setAvBin(0);
    }
}
