/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QSettings>


#include <QCoreApplication>
#include <QMessageBox>
#include <QTimer>

#include "kportsupdateview.h"
#include "kportsicons.h"
#include "kportsglobal.h"
#include "bsdportglobal.h"
#include "portsdirhandler.h"

KPortsUpdateView::KPortsUpdateView(QWidget* parent)
        : QWidget(parent)
{
    setupUi(this);

    reset();

    connect(QBsdPort::global->portsDirHandler(),
                SIGNAL(clearPortsnapState(int, const QString&)),
            this,
                SLOT(clearPortsnapState(int, const QString&)));

    connect(QBsdPort::global->portsDirHandler(),
                SIGNAL(clearPortsState(int, const QString&)),
            this,
                SLOT(clearPortsState(int, const QString&)));

    connect(QBsdPort::global->portsDirHandler(),
                SIGNAL(portsnapState(int, const QString&)),
            this,
                SLOT(portsnapState(int, const QString&)));
                
    connect(commitButton,   SIGNAL(clicked()),
            this,           SLOT(commitButtonClicked()));
    connect(cancelButton,   SIGNAL(clicked()),
            this,           SLOT(cancelButtonClicked()));


}

void KPortsUpdateView::reset()
{
    QSettings settings("kports", "kports");
    int s = settings.value("icons/toolbariconsize", 32).toInt();
    QSize ts(s,s);

    appLabel->setText("<span style=\" font-size:14pt; font-weight:600; "
    "color:#e85848;\">" +
    QCoreApplication::applicationName() + " " +
    QCoreApplication::applicationVersion() +
    "</span>");
    commitButton->setIcon(KPortsIcons::iconLoader.loadIcon("view-refresh"));
    commitButton->setIconSize(ts);
    cancelButton->setIcon(KPortsIcons::iconLoader.loadIcon("go-previous"));
    cancelButton->setIconSize(ts);
    cancelButton->setText(tr("go back"));

    rmPortsnapIcon->clear();
    mvPortsnapIcon->clear();
    rmPortsIcon->clear();
    mvPortsIcon->clear();
    
//     rmPortsnapText->clear();
//     mvPortsnapText->clear();
//     rmPortsText->clear();
//     mvPortsText->clear();
    mvPortsnapText->setText(tr("moving out of the way"));
    rmPortsnapText->setText(tr("removing old directory"));
    mvPortsText->setText(tr("moving out of the way"));
    rmPortsText->setText(tr("removing old directory"));
    
    rmPortsnapIcon->hide();
    mvPortsnapIcon->hide();
    rmPortsIcon->hide();
    mvPortsIcon->hide();

    rmPortsnapText->hide();
    mvPortsnapText->hide();
    rmPortsText->hide();
    mvPortsText->hide();

//     portsnapDirSpacer->hide();
//     portsDirSpacer->hide();

    icon->setPixmap(QPixmap(":/custom_artwork/custom_artwork/update_image.png"));

    progressLabel->hide();
    progressBar->hide();
    progressIcon->hide();

//     umaIsRunning = false;
}

void KPortsUpdateView::commitButtonClicked()
{
    cancelButton->setText(tr("cancel"));
    cancelButton->setIcon(KPortsIcons::iconLoader.loadIcon("dialog-cancel"));

    commitButton->setEnabled(false);
    clearPortsnap->setEnabled(false);
    clearAndRefetch->setEnabled(false);

    QMetaObject::invokeMethod(QBsdPort::global->portsDirHandler(),
                              "start",
                              Qt::QueuedConnection,
                              Q_ARG(bool, clearPortsnap->isChecked()),
                              Q_ARG(bool, clearAndRefetch->isChecked()));
        
/*
    progressLabel->setText(tr("<b>Update running...</b>"));
    progressIcon->setPixmap(KPortsIcons::iconLoader.loadIcon(
                            "chronometer").pixmap(22,22));
    progressIcon->show();
    progressLabel->show();*/
}

void KPortsUpdateView::clearPortsnapState(int state, const QString& log)
{
    QSettings settings("kports", "kports");
    int s = settings.value("icons/toolbariconsize", 32).toInt();
    QSize size(s,s);
    mvPortsnapIcon->show();
    mvPortsnapText->show();

    switch(state)
    {
        case  1: mvPortsnapIcon->setPixmap(KPortsIcons::iconLoader.loadIcon(
                            "chronometer").pixmap(22,22));

                 break;
        case -1:{
                mvPortsnapIcon->setPixmap(KPortsIcons::iconLoader.loadIcon(
                            "edit-delete").pixmap(22,22));
                 QMessageBox msgBox;
                 msgBox.setWindowTitle(tr("KPorts - Error"));
                 msgBox.setStandardButtons(QMessageBox::Ok);
                 msgBox.setIconPixmap(KPortsIcons::iconLoader.loadIcon(
                                    "dialog-error").pixmap(size));
                 msgBox.setText(tr("Error moving the portsnap-Directory"));
                 msgBox.setDetailedText(log);
                 msgBox.exec();
                 reset();
                 }break;
        case  2: mvPortsnapIcon->setPixmap(KPortsIcons::iconLoader.loadIcon(
                            "dialog-ok-apply").pixmap(22,22));
                 rmPortsnapIcon->show();
                 rmPortsnapText->show();
                 rmPortsnapIcon->setPixmap(KPortsIcons::iconLoader.loadIcon(
                            "chronometer").pixmap(22,22));
                 break;
        case -2:{
                rmPortsnapIcon->setPixmap(KPortsIcons::iconLoader.loadIcon(
                            "edit-delete").pixmap(22,22));
                 QMessageBox msgBox;
                 msgBox.setWindowTitle(tr("KPorts - Warning"));
                 msgBox.setStandardButtons(QMessageBox::Ok);
                 msgBox.setIconPixmap(KPortsIcons::iconLoader.loadIcon(
                                    "dialog-warning").pixmap(size));
                 msgBox.setText(tr("Error removing the portsnap-Directory.\n"
                                   "This is not fatal, as it was moved out of "
                                   "the way, but you might want to remove it "
                                   "manually."));
                 msgBox.setDetailedText(log);
                 msgBox.exec();                 
                 } break;
        case  3: rmPortsnapIcon->setPixmap(KPortsIcons::iconLoader.loadIcon(
                            "dialog-ok-apply").pixmap(22,22));
                 break;
        }
}

void KPortsUpdateView::clearPortsState(int state, const QString& log)
{
    QSettings settings("kports", "kports");
    int s = settings.value("icons/toolbariconsize", 32).toInt();
    QSize size(s,s);
        
    mvPortsIcon->show();
    mvPortsText->show();

    switch(state)
    {
        case  1: mvPortsIcon->setPixmap(KPortsIcons::iconLoader.loadIcon(
                            "chronometer").pixmap(22,22));

                 break;
        case -1:{
                 mvPortsIcon->setPixmap(KPortsIcons::iconLoader.loadIcon(
                            "edit-delete").pixmap(22,22));
                 QMessageBox msgBox;
                 msgBox.setWindowTitle(tr("KPorts - Error"));
                 msgBox.setStandardButtons(QMessageBox::Ok);
                 msgBox.setIconPixmap(KPortsIcons::iconLoader.loadIcon(
                                    "dialog-error").pixmap(size));
                 msgBox.setText(tr("Error moving the ports-Directory"));
                 msgBox.setDetailedText(log);
                 msgBox.exec();
                 reset();
                 } break;
        case  2: mvPortsIcon->setPixmap(KPortsIcons::iconLoader.loadIcon(
                            "dialog-ok-apply").pixmap(22,22));
                 rmPortsIcon->show();
                 rmPortsText->show();
                 rmPortsIcon->setPixmap(KPortsIcons::iconLoader.loadIcon(
                            "chronometer").pixmap(22,22));
                 break;
        case -2: {
                 rmPortsIcon->setPixmap(KPortsIcons::iconLoader.loadIcon(
                            "edit-delete").pixmap(22,22));
                 QMessageBox msgBox;
                 msgBox.setWindowTitle(tr("KPorts - Warning"));
                 msgBox.setStandardButtons(QMessageBox::Ok);
                 msgBox.setIconPixmap(KPortsIcons::iconLoader.loadIcon(
                                    "dialog-warning").pixmap(size));
                 msgBox.setText(tr("Error removing the ports-Directory.\n"
                                   "This is not fatal, as it was moved out of "
                                   "the way, but you might want to remove it "
                                   "manually."));
                 msgBox.setDetailedText(log);
                 msgBox.exec();
                 } break;
        case  3: rmPortsIcon->setPixmap(KPortsIcons::iconLoader.loadIcon(
                            "dialog-ok-apply").pixmap(22,22));
                 break;
        }
}

void KPortsUpdateView::portsnapState(int state, const QString& log)
{
    QSettings settings("kports", "kports");
    int s = settings.value("icons/toolbariconsize", 32).toInt();
    QSize size(s,s);
    progressIcon->show();
    progressLabel->show();

    switch(state)
    {
        case  1:
                progressLabel->setText(tr("<b>Portsnap running...</b>"));
                progressIcon->setPixmap(KPortsIcons::iconLoader.loadIcon(
                                        "chronometer").pixmap(22,22));
                return;
        case -1:{
                progressLabel->setText(tr("<b>Portsnap failed.</b>"));
                progressIcon->setPixmap(KPortsIcons::iconLoader.loadIcon(
                                        "edit-delete").pixmap(22,22));


                QMessageBox msgBox;
                msgBox.setWindowTitle(tr("KPorts - Error"));
                msgBox.setStandardButtons(QMessageBox::Ok);
                msgBox.setIconPixmap(KPortsIcons::iconLoader.loadIcon(
                                   "dialog-error").pixmap(size));
                msgBox.setText(tr("Updating the ports with Portsnap failed."));
                msgBox.setDetailedText(log);
                msgBox.exec();
                reset();
                } break;
        case  2:
                progressLabel->setText(tr("<b>Portsnap successful.</b>"));
                progressIcon->setPixmap(QPixmap(QString::fromUtf8(
                    ":/custom_artwork/custom_artwork/apply_green.png")).scaled(22,22));
                QMetaObject::invokeMethod(QBsdPort::global,
                                          "reset",
                                          Qt::QueuedConnection);
                break;
    }
    commitButton->setIcon(KPortsIcons::iconLoader.loadIcon("view-refresh"));
    commitButton->setIconSize(size);
    cancelButton->setIcon(KPortsIcons::iconLoader.loadIcon("go-previous"));
    cancelButton->setIconSize(size);
    cancelButton->setText(tr("go back"));
    commitButton->setEnabled(state!=2);
    clearPortsnap->setEnabled(true);
    clearAndRefetch->setEnabled(true);
}


void KPortsUpdateView::cancelButtonClicked()
{
    if (cancelButton->text() == tr("go back"))
    {
        emit done();
        commitButton->setEnabled(true);
    }
    else
        QMetaObject::invokeMethod(QBsdPort::global->portsDirHandler(),
                                  "killall",
                                  Qt::QueuedConnection); 
}
