/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "portscollection.h"
#include <math.h>

 QBsdPort* QPortsCollection::portWithOrigin(const  QString& origin,
                                         uint   startPos,
                                         int    *returnPos) const
{
//    QMutexLocker locker(&collectionLOCK);

    if (collection().size() == 0)
        return 0;


    QBsdPort *p = 0;

    unsigned int max_i = collection().size()-1;

    // Binaersuche nach origin
    unsigned int links = startPos;
    unsigned int mitte = startPos+1;//collection().size() / 2;
    unsigned int rechts = max_i;

    forever
    {
        mitte = links + ((rechts - links) / 2);

        if ( (mitte > max_i) ||
             (rechts < links) )
             return 0;

        p = collection().at(mitte);

        if (p->origin()==origin)
        {
            if (returnPos != 0)
                *returnPos = mitte;
            return p;
        }

        if (p->origin() > origin)
            rechts = mitte - 1;
        else
            links = mitte + 1;
    }


//     if (*i)
//         qDebug("iterator shows:" + i->origin() +" for " + origin);
// 
//     TODO replace with a O = log n function
//     for (uint i=0; i < collection().size(); ++i)
//         if (collection().at(i)->origin()==origin)
//             return collection().at(i);
//    emit warning(origin.toLatin1() + " not found");


    return 0;
}


const QBsdPort::QPortList QPortsCollection::portWithPkgName(const QString& name,
                                                          PKGTYPE    pkgtype,
                                                          bool withVersion) const
{
    QString n,v;
    QBsdPort *p;
    QPortList ret;
    bool add;

    if (withVersion)
    {
        n = name.section('-',0,-2);
        v = name.section('-',-1,-1);
    } else
        n = name;

//    QMutexLocker locker(&collectionLOCK);

    for (int i = 0; i < collection().size(); ++i)
    {
        p = collection().at(i);
        add = false;
        if (p->hasAvSrc() && (pkgtype &  AVSRCPKG) )
            if ( (p->avSrc()->name()==n) && ( (!withVersion) ||
                  ( (withVersion) && (p->avSrc()->version()==v)) ) )
                add = true;
        if (!add)
            if (p->hasInst() && (pkgtype & INSTPKG) )
                if ( (p->inst()->name()==n) && ( (!withVersion) ||
                        ( (withVersion) && (p->inst()->version()==v)) ) )
                    add = true;;
        if (!add)
            if (p->hasAvBin() && (pkgtype & AVBINPKG) )
                if ( (p->avBin()->name()==n) && ( (!withVersion) ||
                    ( (withVersion) && (p->avBin()->version()==v)) ) )
                    add = true;

        if (add)
            ret += p;
    }

    return ret;
}

QBsdPort::QPortList QPortsCollection::search(const QString& term,
                                           qint64 mask)
{

    QPortList ret;
    QBsdPort *p;
    QString compTerm;
    int percent = 0,
        percentbuf = 0;
    bool add = false;
    bool hasAvSrcPkg = false,
         hasAvBinPkg = false,
         hasInstPkg  = false;

    emit searching();


    int min = 20,
        max = 0;
    for (int j = 0; j < 20; ++j)
    {
        if ((mask & (qint64)pow(2,j)) && (min > j))
            min = j;
        if ((mask & (qint64)pow(2,j)) && (max < j))
            max = j;
    }

    int j = min;

    for (int i = 0; i < collection().size(); ++i)
    {

        percentbuf = i*100/collection().size();
        if ( percentbuf != percent)
        {
            percent = percentbuf;
            emit searchProgress(percent);
        }

        p = collection().at(i);
        hasAvSrcPkg = p->hasAvSrc();
        hasAvBinPkg = p->hasAvBin();
        hasInstPkg  = p->hasInst();

        j = min;
        add = false;
        while ((j <= max) && (!add))
        {
            if (mask & (qint64)pow(2,j))
                switch(j)
                {
                    case  0: if (hasAvSrcPkg)
                                add = p->avSrc()->name().contains(term,
                                                            Qt::CaseInsensitive);
                             break;
                    case  1: if (hasAvSrcPkg)
                                add = p->avSrc()->comment().contains(term,
                                                            Qt::CaseInsensitive);
                             break;
                    case  2: if (hasAvSrcPkg)
                                add = (p->avSrc()->files().filter(term,
                                               Qt::CaseInsensitive).size() > 0 );
                             break;
                    case  3: if (hasAvSrcPkg)
                                add = p->avSrc()->descr().contains(term,
                                                           Qt::CaseInsensitive);
                             break;

                    case  8: if (hasAvBinPkg)
                                add = p->avBin()->name().contains(term,
                                                            Qt::CaseInsensitive);
                             break;
                    case  9: if (hasAvBinPkg)
                                add = p->avBin()->comment().contains(term,
                                                            Qt::CaseInsensitive);
                             break;

                    case 16: if (hasInstPkg)
                                add = p->inst()->name().contains(term,
                                                            Qt::CaseInsensitive);
                             break;
                    case 17: if (hasInstPkg)
                                add = p->inst()->comment().contains(term,
                                                            Qt::CaseInsensitive);
                             break;
                    case 18: if (hasInstPkg)
                                add = (p->inst()->files().filter(term,
                                               Qt::CaseInsensitive).size() > 0 );
                             break;
                    case 19: if (hasInstPkg)
                                add = p->inst()->descr().contains(term,
                                                            Qt::CaseInsensitive);
                             break;
                    default:
                            add = false;
                }

            ++j;
        }

        if (add)
        {
            emit searchResult(p);
            ret += p;
        }
    }

   emit searchFinished();

   return ret;


}


QBsdPort::QPortList QPortsCollection::search(const QString &term,
                                           int criteria,
                                           int    pkgtype,
                                           bool exact)
{
    /** there are two ways to use this function: either wait for it to
        return a portlist or watch the signals it emits. **/

    QPortList ret;
    QBsdPort *p;
    QString compTerm;
    int percent = 0,
        percentbuf = 0;
    bool add = false;
    bool hasAvSrcPkg = false,
         hasAvBinPkg = false,
         hasInstPkg  = false;


    emit searching();

//    QMutexLocker locker(&collectionLOCK);

    for (int i = 0; i < collection().size(); ++i)
    {
        add = false;
        percentbuf = i*100/collection().size();
        if ( percentbuf != percent)
        {
            percent = percentbuf;
            emit searchProgress(percent);
        }

        p = collection().at(i);

        hasAvSrcPkg = p->hasAvSrc() && (pkgtype & AVSRCPKG);
        hasAvBinPkg = p->hasAvBin() && (pkgtype & AVBINPKG);
        hasInstPkg  = p->hasInst() && (pkgtype & INSTPKG);
        if (criteria & PKGNAME)
        {
            if (hasAvSrcPkg && p->avSrc()->name().contains(term,
                                                           Qt::CaseInsensitive))
                add = true;
            else if (hasAvBinPkg && p->avBin()->name().contains(term,
                                                           Qt::CaseInsensitive))
                add = true;
            else if (hasInstPkg && p->inst()->name().contains(term,
                                                           Qt::CaseInsensitive))
                add = true;
        }
        if (!add && (criteria & CATEGORIES))
        {
            if (hasAvSrcPkg && p->avSrc()->categories().contains(
                                         " " + term + " ", Qt::CaseInsensitive))
                add = true;
        }
        if (!add && (criteria & MAINT))
        {
            if (hasAvSrcPkg && p->avSrc()->maint().contains(term,
                                                           Qt::CaseInsensitive))
                add = true;
            else if (hasAvBinPkg && p->avBin()->maint().contains(term,
                                                           Qt::CaseInsensitive))
                add = true;
        }
        if (!add && (criteria & COMMENT))
        {
            if (hasAvSrcPkg && p->avSrc()->comment().contains(term,
                                                           Qt::CaseInsensitive))
                add = true;
            else if (hasAvBinPkg && p->avBin()->comment().contains(term,
                                                           Qt::CaseInsensitive))
                add = true;
            else if (hasInstPkg && p->inst()->comment().contains(term,
                                                           Qt::CaseInsensitive))
                add = true;
        }

        if (!add && (criteria & DESCR))
        {
            if (hasAvSrcPkg && p->avSrc()->descr().contains(term,
                                                           Qt::CaseInsensitive))
                add = true;
            else if (hasInstPkg && p->inst()->descr().contains(term,
                                                           Qt::CaseInsensitive))
                add = true;
        }

        if (!add && (criteria & FILES))
        {
            if (hasAvSrcPkg && (p->avSrc()->files().filter(term,
                                    Qt::CaseInsensitive).size() > 0 ) )
                add = true;
            else if (hasInstPkg && (p->inst()->files().filter(term,
                                    Qt::CaseInsensitive).size() > 0 ) )
                add = true;
        }

        if (add)
        {
            emit searchResult(p);
            ret += p;
        }
    }

   emit searchFinished();

   return ret;
}
