/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QApplication>
#include "portsdirhandler.h"

void QPortsDirHandler::start(bool _clearPortsnap, bool _clearPorts)
{
    reset();
    
    QString cmd;
    if (_clearPortsnap)
    {
        QStringList outList;

        emit clearPortsnapState(1, clearPortsnapLog);
        cmd = "/bin/mv /var/db/portsnap /var/db/portsnapABOUTTODELETE"
              " && mkdir /var/db/portsnap";
        clearPortsnapLog = tr("COMMAND RUN:\n") + cmd + tr("\nRETURN VALUE: %1"
                              "\nLOG:\n %2");

        int result = RunCmd::fg(cmd, outList);

        clearPortsnapLog = clearPortsnapLog.arg(result);
        clearPortsnapLog = clearPortsnapLog.arg(outList.join("\n"));
        

        if (result==0)
        {
            emit clearPortsnapState(2, clearPortsnapLog);
            cmd = "rm -rf /var/db/portsnapABOUTTODELETE";
            clearPortsnapLog += tr("COMMAND RUN:\n") + cmd + tr("\nRETURN VALUE: %1"
                                   "\nLOG:\n");
            RunCmd::bgOutAtEnd(cmd, this,
                               SLOT(appendToClearPortsnapLog(const QStringList&)),
                               SLOT(rmPortsnapDirDone(int, QProcess::ExitStatus)));
        }else
        {
            emit clearPortsnapState(-1, clearPortsnapLog);
            //emit fail();
            return;
        }
    }

    if (_clearPorts)
    {
        QStringList outList;

        emit clearPortsState(1, clearPortsLog);
        cmd = "/bin/mv /usr/ports /usr/portsABOUTTODELETE && "
              "mkdir /usr/ports";
        clearPortsLog = tr("COMMAND RUN:\n") + cmd + tr("\nRETURN VALUE: %1"
                              "\nLOG:\n %2");

        int result = RunCmd::fg(cmd, outList);

        clearPortsLog = clearPortsLog.arg(result);
        clearPortsLog = clearPortsLog.arg(outList.join("\n"));

        if (result==0)
        {
            emit clearPortsState(2, clearPortsLog);
            cmd = "rm -rf /usr/portsABOUTTODELETE";
            clearPortsnapLog += tr("COMMAND RUN:\n") + cmd + tr("\nRETURN VALUE: %1"
                                   "\nLOG:\n");
            procs.append(RunCmd::bgOutAtEnd(cmd, this,
                              SLOT(appendToClearPortsLog(const QStringList&)),
                              SLOT(rmPortsDirDone(int, QProcess::ExitStatus))));
        } else
        {
            emit clearPortsState(-1, clearPortsLog);
            //emit fail();
            return;
        }
    }

    emit portsnapState(1, portsnapLog);
//     cmd = "uma -v " + QString::number(QApplication::applicationPid()) +
    cmd = PREFIX "/sbin/portsnap_patched fetch " 
         + QString(_clearPorts ? "extract" : "update");
    portsnapLog += tr("COMMAND RUN:\n") + cmd + tr("\nRETURN VALUE: %1"
                      "\nLOG:\n");
    procs.append(RunCmd::bg(cmd, this, SLOT(portsnapOutput(const QString&)),
                          SLOT(portsnapFinished(int, QProcess::ExitStatus))));

}


void QPortsDirHandler::rmPortsnapDirDone(int exitCode,
                                         QProcess::ExitStatus exitStatus)
{
    clearPortsnapLog = clearPortsnapLog.arg(exitCode);
    if ( (exitStatus) || (exitCode) )
        emit clearPortsnapState(-2, clearPortsnapLog);
    else
        emit clearPortsnapState(3, clearPortsnapLog);
    procs.removeAll(qobject_cast<QProcess*>(sender()));
}

void QPortsDirHandler::rmPortsDirDone(int exitCode,
                                         QProcess::ExitStatus exitStatus)
{
    clearPortsLog = clearPortsLog.arg(exitCode);
    if ( (exitStatus) || (exitCode) )
        emit clearPortsState(-2, clearPortsLog);
    else
        emit clearPortsState(3, clearPortsLog);
    procs.removeAll(qobject_cast<QProcess*>(sender()));
}

void QPortsDirHandler::portsnapFinished(int exitCode,
                                         QProcess::ExitStatus exitStatus)
{
    if (exitCode != BGRunnerBase::FAILEDTOSTART)
        portsnapLog = portsnapLog.arg(exitCode);
    else
    {
        portsnapLog = portsnapLog.arg('?');
        portsnapLog.append(tr("The command failed to start. Maybe the program does not exist "
                              "or is not executable."));
    }

    if ( (exitStatus) || (exitCode) )
        emit portsnapState(-1, portsnapLog);
    else
        emit portsnapState(2, portsnapLog);
    procs.removeAll(qobject_cast<QProcess*>(sender()));
}

void QPortsDirHandler::killall()
{
    foreach(QProcess* p, procs)
        if (p)
            p->terminate();
        
    foreach(QProcess* p, procs)
        if (p)
            if (!p->waitForFinished(1000))
                p->kill();

    //TODO reinplace moved dirs if they exist
}
