/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "runcmd.h"

RunCmd* RunCmd::m_handler = 0;

int RunCmd::fg(const QString& cmd, QStringList& output)
{
    FILE*   pipe;
    char    buffer[10001] = "";
    QString qbuffer("");
    //TODO remove frame, not neccessary
//    QString realcmd = "/bin/sh -c \'" + cmd + " 2>&1\'";
    QString realcmd = cmd + " 2>&1";

    if( (pipe = popen( realcmd.toLatin1(), "r" )) == NULL )
    {
           qWarning(QString("runCmd() : could not open pipe for command:\n" +
                    cmd).toLatin1());
           output = QStringList();
           return -2;
    }

    while (!feof(pipe))
    {
        fgets( buffer, 10000, pipe );
        qbuffer = buffer;
        qbuffer.remove(qbuffer.length()-1,1);
        qbuffer.remove('\n');
        output.append(qbuffer);
    }


    /// remove trailing empty lines
    while ( (!output.isEmpty()) &&
            ( (output.last() == "") || (output.last() == " ") ) )
        output.removeLast();

    /// work around the double last line
    if (output.count() >1 )
        if (output[output.count()-1] == output[output.count()-2])
            output.removeLast();

    if (output.isEmpty())
        output.append("");

    return pclose(pipe);
}

// does the same as the above but with qprocess
// keeping 2 version because I think the above is faster, however it does not
// work in main.cpp
int RunCmd::fgQ(const QString& cmd, QStringList& output)
{

    QProcess p;
    char buf[1024];
    QString line;
    qint64 lineLength = 0;
    
    p.setProcessChannelMode(QProcess::MergedChannels);
    p.start(cmd, QIODevice::ReadOnly);

    if (!p.waitForStarted(5000))
        return BGRunnerBase::FAILEDTOSTART;

    while ((!p.waitForFinished(0)) || (p.canReadLine()) )
    {
        if (p.canReadLine())
        {
            lineLength = p.readLine(buf, sizeof(buf));
            if (lineLength != -1)
            {
                line = buf;
                output.append(line.remove('\n'));
            }
        }
    }
    return p.exitCode();
}

BGRunnerInstantOut* RunCmd::bg(const QString& cmd,
                const QObject* rec,
                const char* recSlot, /// Slot to receive the output
                const char* recDoneSlot = 0) ///Slot to receaive
{
    BGRunnerInstantOut* runner = new BGRunnerInstantOut(handler());

    runner->setCmd(cmd);
    runner->setProcessChannelMode(QProcess::MergedChannels);

    connect(runner, SIGNAL(output(const QString&)),
            rec,    recSlot);

    if (recDoneSlot)
    {
        connect(runner, SIGNAL(failedToStart(int, QProcess::ExitStatus)),
                rec,    recDoneSlot);
        connect(runner, SIGNAL(finished(int, QProcess::ExitStatus)),
                rec,    recDoneSlot);
    }

    connect(runner, SIGNAL(failedToStart(int, QProcess::ExitStatus)),
            handler(), SLOT(bgRunnerFinished(int, QProcess::ExitStatus)));
    connect(runner, SIGNAL(finished(int, QProcess::ExitStatus)),
            handler(), SLOT(bgRunnerFinished(int, QProcess::ExitStatus)));

    runner->start();
    return runner;
}

BGRunnerSingleOut * RunCmd::bgOutAtEnd(const QString& cmd,
                                       const QObject* rec,
                                       const char* recSlot,
                                       const char* recDoneSlot = 0)
{
    BGRunnerSingleOut *runner = new BGRunnerSingleOut(handler());

    runner->setProcessChannelMode(QProcess::MergedChannels);
    runner->setCmd(cmd);

    connect(runner, SIGNAL(outputAll(const QStringList&)),
            rec,    recSlot);

    if (recDoneSlot)
    {
        connect(runner, SIGNAL(failedToStart(int, QProcess::ExitStatus)),
                rec,    recDoneSlot);
        connect(runner, SIGNAL(finished(int, QProcess::ExitStatus)),
                rec,    recDoneSlot);
    }

    connect(runner, SIGNAL(failedToStart(int, QProcess::ExitStatus)),
            handler(), SLOT(bgRunnerFinished(int, QProcess::ExitStatus)));
    connect(runner, SIGNAL(finished(int, QProcess::ExitStatus)),
            handler(), SLOT(bgRunnerFinished(int, QProcess::ExitStatus)));

    runner->start();
    return runner;
}

//non-static (slot)
void RunCmd::bgRunnerFinished(int exitCode, QProcess::ExitStatus exitStatus)
{
    BGRunnerBase *sr = qobject_cast<BGRunnerBase *>(sender());
    if (!sr)
        return;
    if (exitStatus)
        qWarning(QString("The following call with PID " +
                         QString::number(sr->pid()) + "crashed: " + sr->cmd() +
                         "\n Thats bad.").toLatin1());
    else if (exitCode)
    {
        if (exitCode == BGRunnerBase::FAILEDTOSTART)
             qWarning(QString("The following call with PID " +
                    QString::number(sr->pid()) + " failed to start, maybe the program "
                    "is missing or not executable: " + sr->cmd()).toLatin1());
        else
             qWarning(QString("The following call with PID " +
                    QString::number(sr->pid()) + " returned non-zero value(" +
                    QString::number(exitCode) + "): " + sr->cmd() +
                    "\n That might indicate a problem.").toLatin1());
    }
    sender()->deleteLater();
}

//------ BGRunner



void BGRunnerBase::readStdOut()
{
    char buf[1024];
    QString line;
    qint64 lineLength = 0;
    while(canReadLine())
    {
        lineLength = readLine(buf, sizeof(buf));
        if (lineLength != -1)
        {
            //TODO optimize this (only emit OR lbuffer.append needed!)
            line = buf;
            line.remove('\n');
            processOneLine(line);
            
//             processOneLine<typeholder>(line(buf).remove('\n'));
        }
    }
}

void BGRunnerBase::readStdErr()
{
    foreach(QString line,
            QString(readAllStandardError()).split('\n'))
    {
        qWarning(readAllStandardError());
    }
}
