/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __RUNCMD_H__
#define __RUNCMD_H__

#include <QProcess>
#include <QtDebug>


//#define FAILEDTOSTART -777


class BGRunnerBase : public QProcess
{
Q_OBJECT
private:
    QString     m_cmd;
    QProcess::ProcessState        m_lastState;

protected:
    virtual void processOneLine(const QString& line)
    {
        //implement in subclass
    }

public:
    BGRunnerBase(QObject* parent=0) : QProcess(parent)
    {
        connect(this,   SIGNAL(readyReadStandardOutput()),
                this,   SLOT(readStdOut()));
        connect(this,   SIGNAL(readyReadStandardError()),
                this,   SLOT(readStdErr()));
        connect(this,   SIGNAL(stateChanged(QProcess::ProcessState)),
                this,   SLOT(newState(QProcess::ProcessState)));
    }

    QString cmd() const { return m_cmd; }
    void setCmd(const QString& newValue) { m_cmd = newValue; }

    void start()
    {
        QProcess::start(m_cmd, QIODevice::ReadOnly);
    }

    enum ARTIFICIAL_RETURN_VALUES { FAILEDTOSTART = -777 };

private slots:
    void readStdOut();
    void readStdErr();
    void newState(QProcess::ProcessState state)
    {
        // stupid qt won't emit finished if failed to start so we need to check ourselves
        if ( ((state == 0) && (m_lastState == 1)) || (error() == 0))
            emit failedToStart(FAILEDTOSTART, QProcess::NormalExit);
        qDebug() << "CMD: " << m_cmd.toLatin1() << "\nnew state:"
                  << state << "\nerror: " << error() << "\n";
        m_lastState = state;
    }

signals:
    void failedToStart(int, QProcess::ExitStatus);
};

// emits a signal for every line of output
class BGRunnerInstantOut : public BGRunnerBase
{
Q_OBJECT
private:
    inline void processOneLine(const QString& line)
    {
        emit output(line);
    }

public:
    BGRunnerInstantOut(QObject* parent=0) : BGRunnerBase(parent)
    {
        connect(this,   SIGNAL(finished(int, QProcess::ExitStatus)),
                this,   SLOT(readStdOut()));
    }

signals:
    void output(const QString&);
};

// emits a signal with all output at end
class BGRunnerSingleOut : public BGRunnerBase
{
Q_OBJECT
private:
//     QStringList lbuffer;
//     inline void processOneLine(const QString& line)
//     {
//         lbuffer.append(line);
//     }

public:
    BGRunnerSingleOut(QObject* parent=0)
    // do not call parent constructor because of additional connects
    {
        connect(this,   SIGNAL(finished(int, QProcess::ExitStatus)),
                this,   SLOT(emitOutputAll()));
    }
private slots:
    inline void emitOutputAll()
    {
        emit outputAll(QString(readAllStandardOutput()).split('\n'));
    }
signals:
    void outputAll(const QStringList&);
};

class RunCmd : public QObject
{
Q_OBJECT
private:
    static RunCmd* m_handler;

public:
    RunCmd(QObject* parent = 0)
        : QObject(parent)
     {
     }

     ~RunCmd()
     {
     }

    static RunCmd* handler()
    {
        if (!m_handler)
            m_handler = new RunCmd();
        return m_handler;
    }

    static int fg(const QString& cmd, QStringList& output);
    static int fgQ(const QString& cmd, QStringList& output);
    static BGRunnerInstantOut* bg(const QString& cmd, const QObject* rec,
                                  const char* recSlot, const char* recDoneSlot);
    static BGRunnerSingleOut* bgOutAtEnd(const QString& cmd, const QObject* rec,
                           const char* recSlot,const char* recDoneSlot);
public slots:
    void bgRunnerFinished(int, QProcess::ExitStatus);
};



#endif
